/**************************************************************************************
Author:         David Szybala
Website:        www.davidszybala.com
Alt Website:    www.twistedgerbil.com
Program:        uv_channel_cleaner.ms
Date:           07/24/13
Description:    This scripts will copy/move one map channel to another, delete channels,
                order channels (e.g. if you have channels 1,5,7,15 they will be shfted
                to 1,2,3,4) or color the channels.
***************************************************************************************/

try (destroyDialog uv_channel_cleaner_rollout) catch()


struct UvChannelCleaner
(
    processed_materials = #(),
    
    /**************************************************************************
    Returns true if the 'current_object' does not have any UVW modifiers applied
    **************************************************************************/
    fn validate_modifiers current_object =
    (
        local result = true
        local loop_control = true
        
        for current_modifier in current_object.modifiers while loop_control do
        (
            if (matchPattern ((classOf current_modifier) as string) pattern:"*uvw*" ignorecase:true) do
            (
                result = false
                loop_control = false
            ) --end if
        ) --end for
        
        result --return value
    ), --end fn
    
    /**************************************************************************
    Validates the object 'current_object' has support for channel 'uv_channel' Returns true if valid
    **************************************************************************/
    fn validate_uv_channel current_object uv_channel =
    (
        local result = false
        
        if (classOf current_object) == Editable_Poly do
            result = polyop.getMapSupport current_object uv_channel
        
        if (classOf current_object) == Editable_mesh do
            result = meshop.getMapSupport current_object uv_channel
        
        result --return value
    ), --end fn
    
    /**************************************************************************
    Enables the channel 'uv_channel' for object 'current_object' NOTE: setMapSupport can
    hard fail so keep in try/catch
    **************************************************************************/
    fn enable_channel current_object uv_channel =
    (
        if (classOf current_object) == Editable_Poly do
        (
            try
                polyop.setMapSupport current_object uv_channel true
            catch
                polyop.defaultMapFaces current_object uv_channel
        ) --end if
        
        if (classOf current_object) == Editable_mesh do
        (
            try
                meshop.setMapSupport current_object uv_channel true
            catch
                meshop.defaultMapFaces current_object uv_channel
        ) --end if
    ), --end fn
    
    /**************************************************************************
    Copies one UV channel to another
    **************************************************************************/
    fn copy_uv_channel current_object from_channel to_channel delete_channel =
    (
        this.enable_channel current_object to_channel
        
        if (classOf current_object) == Editable_Poly do
        (
            polyop.setNumMapVerts current_object to_channel (polyop.getNumMapVerts current_object from_channel) keep:false
            polyop.setNumMapFaces current_object to_channel (polyop.getNumMapFaces current_object from_channel) keep:false

            for i = 1 to (polyop.getNumMapVerts current_object to_channel) do 
                polyop.setMapVert current_object to_channel i (polyop.getMapVert current_object from_channel i)

            for i = 1 to (polyop.getNumMapFaces current_object to_channel) do
                polyop.setMapFace current_object to_channel i (polyop.getMapFace current_object from_channel i)
                
            if delete_channel do
                polyop.setMapSupport current_object from_channel false
        ) --end if
            
        if (classOf current_object) == Editable_mesh do
        (
            meshop.setNumMapVerts current_object to_channel (meshop.getNumMapVerts current_object from_channel) keep:false
            meshop.setNumMapFaces current_object to_channel (meshop.getNumMapFaces current_object from_channel) keep:false

            for i = 1 to (meshop.getNumMapVerts current_object to_channel) do 
                meshop.setMapVert current_object to_channel i (meshop.getMapVert current_object from_channel i)

            for i = 1 to (meshop.getNumMapFaces current_object to_channel) do
                meshop.setMapFace current_object to_channel i (meshop.getMapFace current_object from_channel i)
                
            if delete_channel do
                meshop.setMapSupport current_object from_channel false
        ) --end if
    ), --end fn
    
    /**************************************************************************
    Recursive function. Processes all textures of 'current_material' for texture maps. Changes
    'from_channel' to 'to_channel'
    **************************************************************************/
    fn set_sub_texture_channel current_material from_channel to_channel =
    (
        local texture_map = undefined
        
        if (getNumSubTexmaps current_material) > 0 do
        (
            for i = 1 to (getNumSubTexmaps current_material) do
            (
                texture_map = (getSubTexmap current_material i)
                
                if texture_map != undefined do
                (
                    if (isProperty texture_map "Coordinates") do
                    (
                        if texture_map.Coordinates.mapChannel == from_channel do
                            texture_map.Coordinates.mapChannel = to_channel
                    ) --end if
                    
                    if (getNumSubTexmaps texture_map) > 0 do
                        set_sub_texture_channel texture_map from_channel to_channel
                ) --end if
            ) --end for
        ) --end if
    ), --end fn
    
    /**************************************************************************
    Recursive function. Processes all materials of 'current_material' for texture maps. Changes
    'from_channel' to 'to_channel'
    **************************************************************************/
    fn set_sub_material_channel current_material from_channel to_channel =
    (
        local sub_material = undefined
        
        for i = 1 to (getNumSubMtls current_material) do
        (
            sub_material = getSubMtl current_material i
            
            if sub_material != undefined do
            (
                set_sub_texture_channel sub_material from_channel to_channel
            
                if (getNumSubMtls sub_material) > 0 do
                    set_sub_material_channel sub_material from_channel to_channel
            ) --end if
        ) --end for
    ), --end fn
    
    /**************************************************************************
    Changes all texture maps on 'current_object' material from 'from_channel' to 'to_channel'
    **************************************************************************/
    fn set_material_texture_channel current_object from_channel to_channel =
    (
        if current_object.material != undefined do
        (
            if (findItem this.processed_materials current_object.material) == 0 do
            (
                if (getNumSubTexmaps current_object.material) > 0 do
                    set_sub_texture_channel current_object.material from_channel to_channel
                    
                if (getNumSubMtls current_object.material) > 0 do
                    set_sub_material_channel current_object.material from_channel to_channel
            
                append this.processed_materials current_object.material
            ) --end if
        ) --end if
    ), --end fn
    
    /**************************************************************************
    Disables map support for channels 'from_channel' through 'to_channel' on object 'current_object'
    **************************************************************************/
    fn delete_uv_channels current_object from_channel to_channel =
    (
        if (classOf current_object) == Editable_Poly do
        (
            for i = from_channel to to_channel do
            (
                if (polyop.getMapSupport current_object i) do
                    polyop.setMapSupport current_object i false
            ) --end for
        ) --end if
        
        if (classOf current_object) == Editable_mesh do
        (
            for i = from_channel to to_channel do
            (
                if (meshop.getMapSupport current_object i) do
                    meshop.setMapSupport current_object i false
            ) --end for
        ) --end if
    ), --end fn
    
    /**************************************************************************
    Colors vertices for channels 'from_channel' through 'to_channel' on object 'current_object'
    **************************************************************************/
    fn color_uv_vertex current_object from_channel to_channel vertex_color =
    (
        if (classOf current_object) == Editable_Poly do
        (
            for i = from_channel to to_channel do
            (
                if (polyop.getMapSupport current_object i) do
                    polyop.setVertColor current_object i #{1..(polyop.getNumMapVerts current_object i)} vertex_color
            ) --end for
        ) --end if
        
        if (classOf current_object) == Editable_mesh do
        (
            for i = from_channel to to_channel do
            (
                if (meshop.getMapSupport current_object i) do
                    meshop.setVertColor current_object i #{1..(meshop.getNumMapVerts current_object i)} vertex_color
            ) --end for
        ) --end if
    ), --end fn
    
    /**************************************************************************
    Returns a list of all supported map channels
    **************************************************************************/
    fn get_uv_channels current_object =
    (
        local supported_channels = #()
        
        if (classOf current_object) == Editable_Poly do
        (
            for i = 1 to ((polyop.getNumMaps current_object) - 1) do
            (
                if (polyop.getMapSupport current_object i) do
                    append supported_channels i
            ) --end for
        )--end if
        
        if (classOf current_object) == Editable_mesh do
        (
            for i = 1 to ((meshop.getNumMaps current_object) - 1) do
            (
                if (meshop.getMapSupport current_object i) do
                    append supported_channels i
            ) --end for
        )--end if
        
        supported_channels --return value
    ), --end fn
    
    /**************************************************************************
    Orders existing UV channels (e.g. if you have channels 1,5,7,15 they will be shfted to 1,2,3,4) for
    the 'current_object' Can also change texture channels and remove the extra channels
    **************************************************************************/
    fn order_uv_channels current_object change_textures clear_channels:false =
    (
        local uv_channels = this.get_uv_channels current_object
        
        for i = 1 to uv_channels.count do
        (
            if uv_channels[i] != i do
            (
                this.copy_uv_channel current_object uv_channels[i] i true
                
                if change_textures do
                    this.set_material_texture_channel current_object uv_channels[i] i
            )
        ) --end for
        
        if clear_channels do
        (
            if (classOf current_object) == Editable_Poly do
            (
                for i = ((polyop.getNumMaps current_object) - 1) to (uv_channels.count + 1) by -1 do
                    ChannelInfo.ClearChannel current_object i
            ) --end if
            
            if (classOf current_object) == Editable_mesh do
            (
                for i = ((meshop.getNumMaps current_object) - 1) to (uv_channels.count + 1) by -1 do
                    ChannelInfo.ClearChannel current_object i
            ) --end if
            
            collapseStack current_object
        ) --end if
    ) --end fn
) --end struct


(--put script into local context
    global uv_channel_cleaner_rollout
    local uv_channel_cleaner_ini = getDir #plugcfg + "\\UvChannelCleaner.ini"
    local uv_channel_cleaner_rollout_pos
    
    rollout uv_channel_cleaner_rollout "UV Channel Cleaner"
    (
        local cleaner = UvChannelCleaner()
        
        /**********************************************************************
        Validates the 'current_object' can be operated on by the UV cleaner. Checks object class,
        existing uv channel 'uv_channel' and object modifiers
        **********************************************************************/
        fn validate_object current_object uv_channel collapse_modifiers =
        (
            local is_valid = true
                
            if not (cleaner.validate_modifiers current_object) and not collapse_modifiers do
            (
                print ("ERROR: Object has UVW modifiers. Stack must be collapsed: " + current_object.name)
                is_valid = false
            ) --end if
            
            if is_valid do
            (
                if collapse_modifiers then
                (
                    if (classOf current_object) != Editable_Poly and (classOf current_object) != Editable_mesh do
                    (
                        print ("ERROR: Object is not an editable poly or mesh:" + current_object.name)
                        is_valid = false
                    ) --end if
                ) --end if
                else
                (
                    if (classOf current_object.baseObject) != Editable_Poly and (classOf current_object.baseObject) != Editable_mesh do
                    (
                        print ("ERROR: Base object is not an editable poly or mesh:" + current_object.name)
                        is_valid = false
                    ) --end if
                ) --end else
            ) --end if
            
            if is_valid and uv_channel != undefined do
            (
                if not (cleaner.validate_uv_channel current_object uv_channel) do
                (
                    print ("ERROR: Channel " + (uv_channel as string) + " is empty. Cannot copy for object: " + current_object.name)
                    is_valid = false
                ) --end else
            ) --end if
            
            is_valid --return value
        ) --end fn
    
        /**********************************************************************
        UI creation
        **********************************************************************/
        groupBox copy_group "Copy" pos:[5,5] width:200 height:105
            label from_channel_label "From Channel:" pos:[15,25] width:75 height:15
            label to_channel_label "To Channel:" pos:[15,45] width:75 height:15
            spinner copy_from_spinner "" pos:[95,24] width:40 height:15 range:[1,99,1] type:#integer
            spinner copy_to_spinner "" pos:[95,44] width:40 height:15 range:[1,99,2] type:#integer
            checkbox delete_from_checkbox "Delete From Channel After Copy" pos:[15,65] width:180 height:15
            button copy_channel_button "Copy UVs Now" pos:[15,85] width:180 height:20
        
        groupbox reset_group "Delete" pos:[5,115] width:200 height:65
            label delete_channel_label "Channels:" pos:[15,135] width:55 height:15
            spinner delete_from_spinner "" pos:[70,134] width:40 height:15 range:[1,99,2] type:#integer
            label delete_separater_label "-" pos:[115,135] width:5 height:15
            spinner delete_to_spinner "" pos:[125,134] width:40 height:15 range:[1,99,3] type:#integer
            button delete_channel_button "Delete UVs Now" pos:[15,155] width:180 height:20
        
        groupbox color_group "Color" pos:[5,185] width:200 height:65
            label color_channel_label "Channels:" pos:[15,205] width:55 height:15
            spinner color_from_spinner "" pos:[70,204] width:40 height:15 range:[1,99,2] type:#integer
            label color_separater_label "-" pos:[115,205] width:5 height:15
            spinner color_to_spinner "" pos:[125,204] width:40 height:15 range:[1,99,3] type:#integer
            button color_vertex_button "Set Vertex Color Now" pos:[15,225] width:140 height:20
            colorpicker vertex_color "" pos:[160,225] width:35 height:20 color:(color 255 255 255)
    
        groupBox order_group "Order" pos:[5,255] width:200 height:70
            button order_channel_button "Order Channels Now" pos:[15,275] width:180 height:20
            button order_clear_channel_button "Order and Clear Channels Now" pos:[15,300] width:180 height:20 enabled:false
            
        groupBox option_group "Options" pos:[5,330] width:200 height:80
            checkbox selection_only_checkbox "Selection Only" pos:[15,350] width:90 height:15 checked:true
            checkbox collapse_stack_checkbox "Collapse Stack" pos:[15,370] width:95 height:15
            checkbox adjust_texture_checkbox "Adjust Texture/Bitmap Channels" pos:[15,390] width:180 height:15
            
        progressBar progress_bar "" pos:[5,420] width:200 height:10
        label status_label "INFO: Standing by..." pos:[5,435] width:200 height:15
        
        /**********************************************************************
        UI move event handler. Stores the curent position of the UI
        **********************************************************************/
        on uv_channel_cleaner_rollout moved pos do
            setIniSetting uv_channel_cleaner_ini "Dialog" "Position" (pos as string)
        
        /**********************************************************************
        Copy UV event handler. Copies UVs from channel A to B
        **********************************************************************/
        on copy_channel_button pressed do
        (
            local uv_objects = #()
            local error_counter = 0
            
            progress_bar.color = [200, 0, 0]
            progress_bar.value = 0
            
            if selection_only_checkbox.checked and selection.count == 0 do
            (
                status_label.text = "ERROR: No objects selected"
                progress_bar.value = 100
                return false
            ) --end if
            
            if copy_from_spinner.value == copy_to_spinner.value do
            (
                status_label.text = "ERROR: Copy channels are the same"
                progress_bar.value = 100
                return false
            ) --end if
            
            --CRITICAL! Clear previous data!
            free cleaner.processed_materials
            
            status_label.text = "INFO: Copying UVs. Please wait..."
            
            if selection_only_checkbox.checked then
                uv_objects = selection as array
            else
                uv_objects = geometry
            
            for i = 1 to uv_objects.count do
            (
                if (validate_object uv_objects[i] copy_from_spinner.value collapse_stack_checkbox.checked) then
                (
                    if collapse_stack_checkbox.checked do
                        collapseStack uv_objects[i]
                    
                    cleaner.copy_uv_channel uv_objects[i] copy_from_spinner.value copy_to_spinner.value delete_from_checkbox.checked

                    if adjust_texture_checkbox.checked do
                        cleaner.set_material_texture_channel uv_objects[i] copy_from_spinner.value copy_to_spinner.value
                ) --end if
                else
                    error_counter += 1
                
                percent = 100.0 * i / uv_objects.count
                progress_bar.value = percent
                if ((maxVersion())[1] / 1000) > 12 do windows.processPostedMessages()
            ) --end for
            
            progress_bar.color = [0, 200, 0]
            progress_bar.value = 100
            
            if error_counter == 0 then
                status_label.text = "INFO: Copy UVs complete"
            else
            (
                print ("ERROR: " + (error_counter as string) + " error(s) occurred during UV copy")
                status_label.text = ("WARNING: " + (error_counter as string) + " error(s). Check listener")
            ) --end else
        ) --end on
        
        /**********************************************************************
        Delete from spinner event handler. Ensures the delete from channel is not greater than the
        to channel
        **********************************************************************/
        on delete_from_spinner changed the_value do
        (
            if the_value > delete_to_spinner.value do
                delete_to_spinner.value = the_value
        ) --end on
        
        /**********************************************************************
        Delete to spinner event handler. Ensures the delete to channel is not less than the
        from channel
        **********************************************************************/
        on delete_to_spinner changed the_value do
        (
            if the_value < delete_from_spinner.value do
                delete_from_spinner.value = the_value
        ) --end on

        /**********************************************************************
        Delete UV event handler. Deletes the UVs for selected channels
        **********************************************************************/
        on delete_channel_button pressed do
        (
            local uv_objects = #()
            local error_counter = 0
            
            progress_bar.color = [200, 0, 0]
            progress_bar.value = 0
            
            if selection_only_checkbox.checked and selection.count == 0 do
            (
                status_label.text = "ERROR: No objects selected"
                progress_bar.value = 100
                return false
            ) --end if
            
            status_label.text = "INFO: Deleting UVs. Please wait..."
            
            if selection_only_checkbox.checked then
                uv_objects = selection as array
            else
                uv_objects = geometry
            
            for i = 1 to uv_objects.count do
            (
                if (validate_object uv_objects[i] undefined collapse_stack_checkbox.checked) then
                (
                    if collapse_stack_checkbox.checked do
                        collapseStack uv_objects[i]
                    
                    cleaner.delete_uv_channels uv_objects[i] delete_from_spinner.value delete_to_spinner.value
                ) --end if
                else
                    error_counter += 1
                
                percent = 100.0 * i / uv_objects.count
                progress_bar.value = percent
                if ((maxVersion())[1] / 1000) > 12 do windows.processPostedMessages()
            ) --end for
            
            progress_bar.color = [0, 200, 0]
            progress_bar.value = 100
            
            if error_counter == 0 then
                status_label.text = "INFO: Delete UVs complete"
            else
            (
                print ("ERROR: " + (error_counter as string) + " error(s) occurred during UV delete")
                status_label.text = ("WARNING: " + (error_counter as string) + " error(s). Check listener")
            ) --end else
        ) --end on
        
        /**********************************************************************
        Color from spinner event handler. Ensures the color from channel is not greater than the
        to channel
        **********************************************************************/
        on color_from_spinner changed the_value do
        (
            if the_value > color_to_spinner.value do
                color_to_spinner.value = the_value
        ) --end on
        
        /**********************************************************************
        Color to spinner event handler. Ensures the color to channel is not less than the
        from channel
        **********************************************************************/
        on color_to_spinner changed the_value do
        (
            if the_value < color_from_spinner.value do
                color_from_spinner.value = the_value
        ) --end on

        /**********************************************************************
        Color vertex event handler. Colors the vertices for the specified channels
        **********************************************************************/
        on color_vertex_button pressed do
        (
            local uv_objects = #()
            local error_counter = 0
            
            progress_bar.color = [200, 0, 0]
            progress_bar.value = 0
            
            if selection_only_checkbox.checked and selection.count == 0 do
            (
                status_label.text = "ERROR: No objects selected"
                progress_bar.value = 100
                return false
            ) --end if
            
            status_label.text = "INFO: Coloring vertices. Please wait..."
            
            if selection_only_checkbox.checked then
                uv_objects = selection as array
            else
                uv_objects = geometry
            
            for i = 1 to uv_objects.count do
            (
                if (validate_object uv_objects[i] undefined collapse_stack_checkbox.checked) then
                (
                    if collapse_stack_checkbox.checked do
                        collapseStack uv_objects[i]
                    
                    cleaner.color_uv_vertex uv_objects[i] color_from_spinner.value color_to_spinner.value vertex_color.color
                ) --end if
                else
                    error_counter += 1
                
                percent = 100.0 * i / uv_objects.count
                progress_bar.value = percent
                if ((maxVersion())[1] / 1000) > 12 do windows.processPostedMessages()
            ) --end for
            
            progress_bar.color = [0, 200, 0]
            progress_bar.value = 100
            
            if error_counter == 0 then
                status_label.text = "INFO: Set vertex color complete"
            else
            (
                print ("ERROR: " + (error_counter as string) + " error(s) occurred during set vertex color")
                status_label.text = ("WARNING: " + (error_counter as string) + " error(s). Check listener")
            ) --end else
        ) --end on
        
        /**********************************************************************
        Order channel event handler. Orders the existing channels (e.g. if you have channels
        1,5,7,15 they will be shfted to 1,2,3,4)
        **********************************************************************/
        on order_channel_button pressed do
        (
            local uv_objects = #()
            local error_counter = 0
            
            progress_bar.color = [200, 0, 0]
            progress_bar.value = 0
            
            if selection_only_checkbox.checked and selection.count == 0 do
            (
                status_label.text = "ERROR: No objects selected"
                progress_bar.value = 100
                return false
            ) --end if
            
            --CRITICAL! Clear previous data!
            free cleaner.processed_materials
            
            status_label.text = "INFO: Ordering channels. Please wait..."
            
            if selection_only_checkbox.checked then
                uv_objects = selection as array
            else
                uv_objects = geometry
            
            for i = 1 to uv_objects.count do
            (
                if (validate_object uv_objects[i] undefined collapse_stack_checkbox.checked) then
                (
                    if collapse_stack_checkbox.checked do
                        collapseStack uv_objects[i]
                    
                    cleaner.order_uv_channels uv_objects[i] adjust_texture_checkbox.checked
                ) --end if
                else
                    error_counter += 1
                
                percent = 100.0 * i / uv_objects.count
                progress_bar.value = percent
                if ((maxVersion())[1] / 1000) > 12 do windows.processPostedMessages()
            ) --end for
            
            progress_bar.color = [0, 200, 0]
            progress_bar.value = 100
            
            if error_counter == 0 then
                status_label.text = "INFO: Order channels complete"
            else
            (
                print ("ERROR: " + (error_counter as string) + " error(s) occurred during order channels")
                status_label.text = ("WARNING: " + (error_counter as string) + " error(s). Check listener")
            ) --end else
        ) --end on
        
        /**********************************************************************
        Order channel event handler. Orders the existing channels (e.g. if you have channels
        1,5,7,15 they will be shfted to 1,2,3,4) and deletes the extra channels
        **********************************************************************/
        on order_clear_channel_button pressed do
        (
            local uv_objects = #()
            local error_counter = 0
            
            progress_bar.color = [200, 0, 0]
            progress_bar.value = 0
            
            if selection_only_checkbox.checked and selection.count == 0 do
            (
                status_label.text = "ERROR: No objects selected"
                progress_bar.value = 100
                return false
            ) --end if
            
            --CRITICAL! Clear previous data!
            free cleaner.processed_materials
            
            --CRITICAL! Hide the modify panel!
            max create mode
            
            status_label.text = "INFO: Ordering channels. Please wait..."
            
            if selection_only_checkbox.checked then
                uv_objects = selection as array
            else
                uv_objects = geometry
            
            for i = 1 to uv_objects.count do
            (
                if (validate_object uv_objects[i] undefined collapse_stack_checkbox.checked) then
                (
                    collapseStack uv_objects[i]
                    cleaner.order_uv_channels uv_objects[i] adjust_texture_checkbox.checked clear_channels:true
                ) --end if
                else
                    error_counter += 1
                
                percent = 100.0 * i / uv_objects.count
                progress_bar.value = percent
                if ((maxVersion())[1] / 1000) > 12 do windows.processPostedMessages()
            ) --end for
            
            progress_bar.color = [0, 200, 0]
            progress_bar.value = 100
            
            if error_counter == 0 then
                status_label.text = "INFO: Order and clear complete"
            else
            (
                print ("ERROR: " + (error_counter as string) + " error(s) occurred during order channels")
                status_label.text = ("WARNING: " + (error_counter as string) + " error(s). Check listener")
            ) --end else
        ) --end on
        
        /**********************************************************************
        Collapse checkbox event handler. Enables/Disables the order and clean button
        **********************************************************************/
        on collapse_stack_checkbox changed the_state do
            order_clear_channel_button.enabled = the_state
    )--end rollout
    
    uv_channel_cleaner_rollout_pos = execute (getIniSetting uv_channel_cleaner_ini "Dialog" "Position")
    if uv_channel_cleaner_rollout_pos == OK do uv_channel_cleaner_rollout_pos = [100,100]
    if uv_channel_cleaner_rollout_pos.x > sysinfo.desktopsize.x - 210 do
        uv_channel_cleaner_rollout_pos.x = sysinfo.desktopsize.x - 215
    if uv_channel_cleaner_rollout_pos.y > sysinfo.desktopsize.y - 510 do
        uv_channel_cleaner_rollout_pos.y = sysinfo.desktopsize.y - 510
    if uv_channel_cleaner_rollout_pos.x < 0 do uv_channel_cleaner_rollout_pos.x = 0 
    if uv_channel_cleaner_rollout_pos.y < 0 do uv_channel_cleaner_rollout_pos.y = 0
    createDialog uv_channel_cleaner_rollout 210 450 uv_channel_cleaner_rollout_pos.x uv_channel_cleaner_rollout_pos.y
)--end local scope
